lazy val commonSettings = Seq(
  organization := "com.github.xmanu.virtual-classes",

  version := "0.1",

  scalaVersion := "2.11.8",

  scalacOptions ++= Seq("-deprecation", "-feature"),

  resolvers += Resolver.sonatypeRepo("snapshots"),

  addCompilerPlugin("org.scalamacros" % "paradise" % "2.1.0" cross CrossVersion.full)
)

lazy val scalaReflect = Def.setting {
  "org.scala-lang" % "scala-reflect" % scalaVersion.value
}
lazy val scalaCompiler = Def.setting {
  "org.scala-lang" % "scala-compiler" % scalaVersion.value
}

lazy val root = project.in(file("."))
  .aggregate(macros, cards)
  .settings(commonSettings: _*)
  .settings(
    name := "root",

    run := (run in Compile in cards).evaluated
  )

lazy val macros = project.in(file("macros"))
  .settings(commonSettings: _*)
  .settings(
    name := "virtual-classes-macros",

    libraryDependencies += scalaReflect.value,
    libraryDependencies += scalaCompiler.value
  )

lazy val examples = project.in(file("examples"))
  .settings(commonSettings: _*)
  .dependsOn(macros)
  .settings(
    name := "virtual-classes-examples",

    libraryDependencies += "org.scalatest" %% "scalatest" % "3.0.1" % "test"
  )

lazy val cards = project.in(file("cards"))
  .settings(commonSettings: _*)
  .dependsOn(macros)
  .settings(
    name := "virtual-classes-cards"
  )